﻿using System;
using UnityEngine;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Reflection;
using UnityEngine.UI;
using TMPro;
using System.IO;
using UnityEngine.SceneManagement;
using Newtonsoft.Json;

public class SettingRadioButton : MonoBehaviour
{
    public void SetChecked()
    {
        label.color = (value == (bool)field.GetValue(instance) ? Color.green : Color.white);
    }

    public Button button;

    public TextMeshProUGUI label;
    public FieldInfo field;
    public object instance;

    public bool value;
}

class SettingTitle : Attribute
{
    public string title;
    public SettingTitle(string title)
    {
        this.title = title;
    }
}

class SettingDesc : Attribute
{
    public string desc;
    public SettingDesc(string desc)
    {
        this.desc = desc;
    }
}

class ModSettingManager : MonoBehaviour
{

    public static ModSettingManager Load()
    {
        ModSettingManager manager = FindObjectOfType<ModSettingManager>();
        if (manager != null)
        {
            return manager;
        }
        GameObject obj = new GameObject("ModSettingManager");
        
        DontDestroyOnLoad(obj);

        return obj.AddComponent<ModSettingManager>();

    }

    void Awake()
    {
        SceneManager.sceneLoaded += (Scene loaded, LoadSceneMode mode) =>
        {
            if (loaded.name == "Main" || loaded.name == "Menu")
            {
                GameObject temp = new GameObject("temp");
                temp.AddComponent<LoadSettingsAfterSceneChange>().allSettings = allSettings;
            }
        };
    }

    private class LoadSettingsAfterSceneChange : MonoBehaviour
    {
        public Dictionary<string, ModSettings> allSettings;
        void LateUpdate()
        {
            //SettingsGeneratorBehaviour settings = FindObjectOfType<SettingsGeneratorBehaviour>();
            foreach (KeyValuePair<string, ModSettings> keyValue in allSettings)
            {
                Debug.Log(keyValue.Key);
                //keyValue.Value.settings = settings;
                keyValue.Value.CreateButtons();
            }
            Destroy(gameObject);
        }
    }


    public Dictionary<string, ModSettings> allSettings = new Dictionary<string, ModSettings>();
    public void Add(ModSettings modSettings)
    {
        Debug.Log(modSettings);
        Debug.Log(modSettings.metaData.Name);
        allSettings.Add(modSettings.metaData.Name, modSettings);
    }

    public ModSettings Get(string modID)
    {
        ModSettings value;
        if (allSettings.TryGetValue(modID, out value))
        {
            return value;
        }
        throw new KeyNotFoundException("That key does not exist. Polars Bear, you big doofus");
    }
} 

public class ModSettings 
{
    public class BoolSetting
    {

        ModSettings modSettings;
        SettingsGeneratorBehaviour settings;
        FieldInfo field;
        object instance;
        SettingTemplateBehaviour template;
        private SettingRadioButton[] buttons;
        private SettingCategory category;
        private string title;
        private string description;
        public BoolSetting(ModSettings modSettings, SettingCategory category, FieldInfo field, string title, string description = null)
        {
            if (field.FieldType != typeof(bool)) { throw new Exception("Ahem... this is for booleans only..."); }
            this.field = field;
            this.instance = modSettings.instance;
            this.settings = modSettings.settings;
            this.modSettings = modSettings;

            this.category = category;
            this.title = title;
            this.description = description;

            Create();
        }

        public void Create()
        {
            settings = GameObject.FindObjectOfType<SettingsGeneratorBehaviour>();
            SettingAttribute attr = new SettingAttribute(category, title, description ?? String.Empty);
            template = GameObject.Instantiate(settings.SettingTemplatePrefab, settings.Container).GetComponent<SettingTemplateBehaviour>();
            template.name = attr.Title;
            template.Category = attr.Category;
            template.SetTitle(attr.Title);
            template.SetDescription(attr.Description);
            CamCreateButtons();
        }

        private void CreateButtons(Camera cam = null)
        {
            CreateRadioButton(true);
            CreateRadioButton(false);
        }

        private void CamCreateButtons()
        {
            Camera.onPostRender += CreateButtons;
            Camera.onPostRender += ClearOnPostRender;
        }

        private void ClearOnPostRender(Camera cam)
        {
            Camera.onPostRender -= CreateButtons; Camera.onPostRender -= ClearOnPostRender;
        }

        private void CreateRadioButton(bool value)
        {
            Debug.Log(1);
            GameObject buttonPrefab = GameObject.Instantiate(settings.ButtonSettingPrefab, template.ControlContainer);
            SettingRadioButton button = buttonPrefab.AddComponent<SettingRadioButton>();
            SettingButtonBehaviour oldbutton = buttonPrefab.GetComponent<SettingButtonBehaviour>();
            button.button = oldbutton.Button;
            button.label = oldbutton.Label;
            Debug.Log(2);
            GameObject.Destroy(oldbutton);
            button.value = value;
            button.field = field;
            button.instance = instance;
            button.label.text = (value ? "Enabled" : "Disabled");
            button.SetChecked();
            Debug.Log(field.GetValue(instance).ToString() + " is button thing");
            button.button.onClick.AddListener(delegate ()
            {
                field.SetValue(instance, button.value);
                modSettings.WriteSettings();
                foreach (SettingRadioButton btn in template.ControlContainer.GetComponentsInChildren<SettingRadioButton>())
                {
                    btn.SetChecked();
                }
                modSettings.ApplySetting.Invoke(settings, new object[] { true });
            });
        }
    }

    private string settingsPath = "";
    private string imageCachePath = "";
    MethodInfo ApplySetting = typeof(SettingsGeneratorBehaviour).GetMethod("ApplySetting");
    public ModMetaData metaData;
    public object instance;
    public List<BoolSetting> boolSettings = new List<BoolSetting>();
    public SettingsGeneratorBehaviour settings;
    public ModSettings(object instance, string settingsFilename, ModMetaData metaData) 
    {
        this.instance = instance;
        this.metaData = metaData;
        settingsPath = metaData.MetaLocation + "\\" + settingsFilename;
        imageCachePath = metaData.MetaLocation + "\\" + "imageCache";
        GameObject.FindObjectOfType<ModSettingManager>().StartCoroutine(AddFields());    
    }

    IEnumerator AddFields()
    {
        settings = null;
        while (settings.Equals(null))
        {
            settings = GameObject.FindObjectOfType<SettingsGeneratorBehaviour>();
            Debug.Log("NOPE");
            yield return new WaitForEndOfFrame();
        }
        foreach (FieldInfo fi in instance.GetType().GetFields())
        {
            if (fi.FieldType == typeof(bool))
            {
                string title = fi.Name;
                string desc = String.Empty;
                SettingTitle setT = fi.GetCustomAttribute<SettingTitle>();
                if (setT != null) { title = setT.title; }
                SettingDesc setD = fi.GetCustomAttribute<SettingDesc>();
                if (setD != null) { desc = setD.desc; }
                boolSettings.Add(new BoolSetting(this, SettingCategory.General, fi, title, desc));
            }
        }
    }

    public void CreateButtons()
    {
        foreach(BoolSetting set in boolSettings)
        {
            set.Create();
        }
    }

    public T ReadSettings<T>()
    {
        try
        {
            StreamReader reader = new StreamReader(settingsPath);
            string raw = reader.ReadToEnd();
            reader.Close();
            ChangeInstance(JsonConvert.DeserializeObject<T>(raw));
            return (T)instance;
        }
        catch (FileNotFoundException)
        {
            WriteSettings();
            return ReadSettings<T>();
        }
    }

    private void ChangeInstance<T>(T newInstance)
    {
        foreach(FieldInfo fi in newInstance.GetType().GetFields())
        {
            fi.SetValue(instance, fi.GetValue(newInstance));
        }
    }

    public void WriteSettings()
    {
        StreamWriter writer = new StreamWriter(settingsPath);
        writer.Write(JsonConvert.SerializeObject(instance));
        writer.Close();
    }

    /*public void CacheTexture(Texture2D texture)
    {
        byte[] raw = ImageConversion.EncodeToPNG(texture);
        if (!Directory.Exists(imageCachePath))
        {
            Directory.CreateDirectory(imageCachePath);
        }
        StreamWriter writer = new StreamWriter(imageCachePath + "\\" + texture.name + ".png");
        writer.BaseStream.Write(raw, 0, raw.Length);
        writer.Close();
    }

    public Texture2D GetCachedTexture(string textureName)
    {
        bool found = false;
        foreach (string fn in Directory.EnumerateFiles(imageCachePath))
        {
            if (fn == textureName + ".png")
            {
                textureName += ".png";
                found = true;
                break;
            }
        }
        if (!found)
        {
            throw new FileNotFoundException("No such cached texture exists", textureName);
        }
        StreamReader reader = new StreamReader(imageCachePath + "//" + textureName);
        byte[] raw = new byte[(int)reader.BaseStream.Length];
        reader.BaseStream.Read(raw, 0, (int)reader.BaseStream.Length);
        reader.Close();

        Texture2D tex = new Texture2D(2, 2);
        tex.LoadImage(raw);
        return tex;
    }

    public Texture2D[] GetAllCachedTextures()
    {
        List<Texture2D> textures = new List<Texture2D>();
        foreach (string fn in Directory.EnumerateFiles(imageCachePath))
        {
            StreamReader reader = new StreamReader(imageCachePath + "//" + fn);
            byte[] raw = new byte[(int)reader.BaseStream.Length];
            reader.BaseStream.Read(raw, 0, (int)reader.BaseStream.Length);
            reader.Close();

            Texture2D tex = new Texture2D(2, 2);
            tex.LoadImage(raw);

            textures.Add(tex);
        }

        return textures.ToArray();
    }*/
}

