﻿using System;
using System.Collections;
using System.Reflection;
using UnityEngine.UI;
using System.Collections.Generic;
using UnityEngine;

namespace Mod
{
    public class DiscoSyringe : SyringeBehaviour
    {
        public const string name = "​<b></b>" + "Disco Syringe";
        public static void Load()
        {
            ModAPI.RegisterLiquid(DiscoSyringeSerum.ID, new DiscoSyringeSerum());

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),

                    NameOverride = name,
                    DescriptionOverride = "Syringe that makes the user a disco ball... person.",

                    CategoryOverride = ModAPI.FindCategory("Polars Bear"),
                    AfterSpawn = (Instance) => { Destroy(Instance.GetComponent<SyringeBehaviour>()); Instance.GetOrAddComponent<DiscoSyringe>(); Instance.GetOrAddComponent<DiscoSyringeSerum.DiscoLightBehaviour>(); }
                }
            );
            Mod.nailgun.SyringeNail(name, typeof(DiscoSyringe));
        }
        public override string GetLiquidID() => DiscoSyringeSerum.ID;

        FieldInfo fi;
        LightSprite light;
        Liquid liquid;
        void Awake()
        {
            base.Awake();
            liquid = Liquid.GetLiquid(GetLiquidID());
            fi = typeof(BloodContainer).GetField("isComputedColourCached", BindingFlags.NonPublic | BindingFlags.Instance);
            light = ModAPI.CreateLight(transform, liquid.Color, .4f, 1.5f);
            light.gameObject.layer = gameObject.layer;
        }
        void Update()
        {
            base.Update();
            light.Color = liquid.Color;
            fi.SetValue(this, false);
        }

        void OnRenderObject()
        {
            if (light.gameObject.layer != 15)
            {
                light.gameObject.layer = 15;
                light.Radius = 2f;
                light.Brightness = 3f;
            }
        }
        public class DiscoSyringeSerum : Liquid
        {
            public const string ID = "​<b></b>" + "DISCO SERUM";
            public DiscoSyringeSerum()
            {
                Color = UnityEngine.Random.ColorHSV();
                ModAPI.Register<LiquidColorChangingBehaviour>();
            }
            public override void OnEnterLimb(LimbBehaviour limb)
            {
                limb.gameObject.GetOrAddComponent<ColorChangingBehaviourWithLiquid>();
            }
            public override void OnExitContainer(BloodContainer container)
            {
                if (container.gameObject.TryGetComponent<LimbBehaviour>(out _)) { return; }
                Destroy(container.GetComponent<DiscoLightBehaviour>());
            }
            public override void OnUpdate(BloodContainer container) { }
            public override void OnEnterContainer(BloodContainer container)
            {
                if (container.gameObject.TryGetComponent<LimbBehaviour>(out _)) { return; }
                container.gameObject.GetOrAddComponent<DiscoLightBehaviour>().container = container;
            }

            public class DiscoLightBehaviour : MonoBehaviour
            {
                LightSprite light;
                Liquid liquid;
                public BloodContainer container;
                FieldInfo fi;
                void Awake()
                {
                    liquid = Liquid.GetLiquid(DiscoSyringeSerum.ID);
                    light = ModAPI.CreateLight(transform, liquid.Color, 7f, 5f);
                    fi = typeof(BloodContainer).GetField("isComputedColourCached", BindingFlags.NonPublic | BindingFlags.Instance);
                }

                void Update()
                {
                    light.Color = liquid.Color;
                    fi.SetValue(container, false);
                }

            }

            private class ColorChangingBehaviourWithLiquid : MonoBehaviour
            {
                Liquid liquid;
                LimbBehaviour limb;
                LightSprite light;
                void Awake()
                {
                    liquid = Liquid.GetLiquid(ID);
                    
                    light = ModAPI.CreateLight(transform, liquid.Color,brightness:0.25f);
                    TryGetComponent<LimbBehaviour>(out limb);
                }

                void OnWillRenderObject()
                {
                    if (limb) { limb.Color = liquid.Color; }
                    light.Color = liquid.Color;
                }

                void OnDestroy()
                {
                    Destroy(light.gameObject);
                }
            }
            private class LiquidColorChangingBehaviour : MonoBehaviour
            {
                Liquid liquid;
                Color newColor;
                Color oldColor;
                float interp = 0f;
                float counter = 0f;
                FieldInfo pi;
                Sprite spr = null;
                void Awake()
                {
                    liquid = Liquid.GetLiquid(ID);
                    pi = typeof(CatalogBehaviour).GetField("items", BindingFlags.NonPublic | BindingFlags.Instance);
                }

                void Update()
                {
                    if (interp == 0f)
                    {
                        oldColor = liquid.Color;
                        newColor = UnityEngine.Random.ColorHSV(0f,1f, 0.75f, 0.75f, 0.75f, 0.75f);
                    }
                    else if (interp >= 1f)
                    {
                        interp = 0f;
                        return;
                    }
                    interp += Time.deltaTime * 2;
                    counter += Time.deltaTime;
                    liquid.Color = UnityEngine.Color.Lerp(oldColor, newColor, Math.Min(interp, 1f));
                    if (counter > 0.5f && Mod.settings.animatedCatalog)
                    {
                        
                        foreach (ItemButtonBehaviour it in (System.Collections.Generic.List<ItemButtonBehaviour>)pi.GetValue(CatalogBehaviour.Main))
                        {
                            if (it.Item.name == DiscoSyringe.name)
                            {
                                if (spr)
                                {
                                    Destroy(spr);
                                }
                                spr = Mod.nailgun.SyringeNail(DiscoSyringe.name, typeof(DiscoSyringe));
                                it.GetComponent<Image>().sprite = spr;
                                break;
                            }
                        }
                        counter = 0f;
                    }
                    
                }
            }
        }

    }
}

//Author: Polars Bear
