﻿using System;
using System.Collections;
using System.Reflection;
using UnityEngine;

namespace Mod
{
	public class FriendlynessSyringe : SyringeBehaviour
	{
		public const string name = "​<b></b>" + "Gorse Friendlyness Syringe";
		public static void Load()
		{
			GameObject dummy = new GameObject("heart_particle_pb");
			Shader sh = Shader.Find("Sprites/Default");



			Material mat = new Material(sh);
			mat.mainTexture = ModAPI.LoadTexture("sprites/heart.png");
			SpriteRenderer ren = dummy.AddComponent<SpriteRenderer>();
			ren.material = mat;
			ren.enabled = false;


			ModAPI.RegisterLiquid(FriendlySerum.ID, new FriendlySerum());

			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),

					NameOverride = name,
					DescriptionOverride = "Syringe that makes Gorses friendly.",

					CategoryOverride = ModAPI.FindCategory("Polars Bear"),
					AfterSpawn = (Instance) => { Destroy(Instance.GetComponent<SyringeBehaviour>()); Instance.GetOrAddComponent<FriendlynessSyringe>(); }
				}
			);
			Mod.nailgun.SyringeNail(name, typeof(FriendlynessSyringe));
		}
		public override string GetLiquidID() => FriendlySerum.ID;
		public class FriendlySerum : Liquid
		{
			public const string ID = "<b></b>" + "GORSE FRIENDLYNESS SERUM";
			public FriendlySerum()
			{
				Color = new UnityEngine.Color(0.4f, 0.6f, 0.4f);
			}
			public override void OnEnterLimb(LimbBehaviour limb)
			{
				if (limb.RoughClassification != LimbBehaviour.BodyPart.Head || limb.SpeciesIdentity != "Gorse")
				{
					return;
				}
				GameObject attacksys = limb.gameObject.transform.GetChild(0).gameObject;
				//Destroy(attacksys.GetComponent<GorseDroolingBehaviour>());
				FriendlyGorseBehaviour friend = attacksys.GetOrAddComponent<FriendlyGorseBehaviour>();


			}

			public override void OnExitContainer(BloodContainer container)
			{
				/*LimbBehaviour limb = container.GetComponent<LimbBehaviour>();
				Debug.Log(limb);
				Debug.Log(limb == null);
				if (limb == null || limb.SpeciesIdentity != "Gorse" || limb.RoughClassification != LimbBehaviour.BodyPart.Head) {
					return;
				}
				Debug.Log("PB1");
				
				container.gameObject.GetComponent<FriendlyGorseBehaviour>().StopPlay();
				Debug.Log("PB2");
				Destroy(container.GetComponent<FriendlyGorseBehaviour>());
				Debug.Log("PB3");
				container.gameObject.GetOrAddComponent<GorseDroolingBehaviour>();
				Debug.Log("PB4");*/ //Did not work... for some reason it only ever left the legs...
			}

			public override void OnUpdate(BloodContainer container) { }
			public override void OnEnterContainer(BloodContainer container) { }

			private class FriendlyGorseBehaviour : MonoBehaviour
			{
                System.Random rng = new System.Random();
				private void Start()
				{
					this.buffer = new Collider2D[32];
				}
				private void FixedUpdate()
				{
					this.step++;
					if (this.step > 30)
					{
						this.isAlive = limb.Person.IsAlive();
						if (this.isAlive && limb.IsCapable)
						{
							this.AttackNearby();
						}
					}
				}
				private void AttackNearby()
				{
					this.step = 0;
					int num = Physics2D.OverlapCircleNonAlloc(base.transform.position, this.Range, this.buffer, this.CanSee);
					if (num == 0)
					{
						return;
					}
					buffer = rng.Shuffle(buffer);
					Collider2D collider2D = null;
					foreach ( Collider2D col in this.buffer)
                    {
						LimbBehaviour l;
						if (col.TryGetComponent<LimbBehaviour>(out l))
						{
							bool flag = false;
							foreach (DamagePoint point in l.SkinMaterialHandler.damagePoints)
                            {
								if (point.Intensity > 0.01f)
                                {
									flag = true;
									break;
                                }
                            }
							if (l.Person != limb.Person && (Mathf.Round(l.Health) < Mathf.Round(l.InitialHealth) || flag))
							{
								collider2D = col;
								break;
							}
						}

                    }
					if (!collider2D) { return; }

					Vector3 v = base.transform.position - collider2D.transform.position;
					Utils.LaserHit laserHit = Utils.MaterialAwareRaycast(collider2D.transform.position, v, this.Range, this.CanSee, 8, 0);
					if (!laserHit.physicalBehaviour || !(laserHit.physicalBehaviour.transform.root == base.transform.root))
					{
						return;
					}
					limb.Person.Wince(15f);
					this.Fire(-v.normalized, v.magnitude * this.Force);
				}
				private void Fire(Vector3 direction, float force)
				{
					limb.PhysicalBehaviour.PlayClipOnce(this.FireClip.PickRandom<AudioClip>());
					GameObject gameObject = GameObject.Instantiate(Projectile, transform.position, Quaternion.identity);
					gameObject.SetActive(true);
					gameObject.GetComponent<Rigidbody2D>().AddForce(direction * force, ForceMode2D.Impulse);
					foreach (Collider2D b in base.transform.root.GetComponentsInChildren<Collider2D>())
					{
						IgnoreCollisionStackController.IgnoreCollisionSubstituteMethod(gameObject.GetComponent<Collider2D>(), b, true);
					}
				}


				public LayerMask CanSee;

				private int step = 0;

				private bool isAlive;

				private Collider2D[] buffer;

				public GameObject Projectile;

				public AudioClip[] FireClip;

				public float Range;

				public float Force;

				private ParticleSystem ps;
				private LimbBehaviour limb;
				private bool hasDied = false;

				void Awake()
				{
					limb = transform.parent.GetComponent<LimbBehaviour>();
					ps = gameObject.GetComponent<ParticleSystem>();
					ParticleSystemRenderer psr = gameObject.GetComponent<ParticleSystemRenderer>();
					psr.material = GameObject.Find("heart_particle_pb").GetComponent<SpriteRenderer>().material;

					var size = ps.sizeOverLifetime;
					size.sizeMultiplier = 10f;
					var main = ps.main;
					main.loop = true;
					main.maxParticles = 25;
					main.startLifetime = 2f;
					main.startColor = new UnityEngine.Color(1f, 1.0f, 1f);
					main.gravityModifier = -.1f;
					var em = ps.emission;
					em.enabled = true;
					em.rateOverTime = 2f;
					ps.Play();
					ps.Emit(5);
					
					Range *= 2f;
					GorseDroolingBehaviour gb = GetComponent<GorseDroolingBehaviour>();
					Projectile = Instantiate(gb.Projectile);
					Range = gb.Range * 2f;
					Force = gb.Force;
					FireClip = gb.FireClip;
					CanSee = gb.CanSee;
					Destroy(gb);
					Projectile.SetActive(false);
					FriendlyGorseProjectileBehaviour pb = Projectile.GetOrAddComponent<FriendlyGorseProjectileBehaviour>();
					pb.DisperseEffect = Projectile.GetComponent<GorseProjectileBehaviour>().DisperseEffect;
					Destroy(Projectile.GetComponent<GorseProjectileBehaviour>());

				}

				void Update()
                {
					if (!limb.Person.IsAlive())
                    {
						StopPlay();
						hasDied = true;
                    }
					else if (hasDied)
                    {
						ps.Play();
					}
                }

				public void StopPlay()
				{
					ps.Stop();
				}

				private class FriendlyGorseProjectileBehaviour : MonoBehaviour
                {
					
					void Awake()
                    {
						Rigidbody2D = GetComponent<Rigidbody2D>();
                    }
					private void OnCollisionEnter2D(Collision2D collision)
					{
						LimbBehaviour limby;
						if (collision.transform && collision.transform.TryGetComponent<LimbBehaviour>(out limby))
						{
							limby.Wince(this.Damage);
							limby.Damage(this.Damage);
							foreach (LimbBehaviour l in limby.Person.Limbs)
							{
								if (!l.IsAndroid)
								{
									l.Health = Math.Min(l.InitialHealth, l.Health + 5f);
									if (UnityEngine.Random.value > 0.25f)
									{
										l.CirculationBehaviour.HealBleeding();
										l.HealBone(); l.LungsPunctured = false;
									}
									l.Numbness = Math.Max(0f, l.Numbness - 0.75f);
									l.CirculationBehaviour.IsPump = l.CirculationBehaviour.WasInitiallyPumping;
									l.CirculationBehaviour.BloodFlow = Math.Min(1f, l.CirculationBehaviour.BloodFlow + 0.75f);
									l.CirculationBehaviour.AddLiquid(l.GetOriginalBloodType(), Math.Max(Mathf.Min(0.75f, 1f - l.CirculationBehaviour.GetAmount(l.GetOriginalBloodType())), 0f));
									l.BruiseCount = (ushort)Math.Max(0, l.BruiseCount - 10);
									l.CirculationBehaviour.GunshotWoundCount = (ushort)Math.Max(0, l.CirculationBehaviour.GunshotWoundCount - 10);
									l.CirculationBehaviour.StabWoundCount = (ushort)Math.Max(0, l.CirculationBehaviour.StabWoundCount - 10);
									if (l.RoughClassification == LimbBehaviour.BodyPart.Head)
									{
										l.Person.Consciousness = Math.Min(1f, l.Person.Consciousness + 0.75f);
										l.Person.ShockLevel = Math.Max(0f, l.Person.ShockLevel - 0.75f);
										l.Person.PainLevel = Math.Max(0f, l.Person.PainLevel - 0.75f);
										l.Person.OxygenLevel = Math.Min(1f, l.Person.OxygenLevel + 0.75f); ;
										l.Person.AdrenalineLevel = Math.Min(1f, l.Person.AdrenalineLevel + 0.75f); ;
									}
									if (l.PhysicalBehaviour.BurnProgress > 0)
									{
										l.PhysicalBehaviour.BurnProgress = Math.Max(l.PhysicalBehaviour.BurnProgress - 0.75f, 0f);
									}
									if (l.SkinMaterialHandler.AcidProgress > 0)
									{
										l.SkinMaterialHandler.AcidProgress = Math.Max(l.SkinMaterialHandler.AcidProgress - 0.75f, 0f);
									}
									if (l.SkinMaterialHandler.RottenProgress > 0)
									{
										l.SkinMaterialHandler.RottenProgress = Math.Max(l.SkinMaterialHandler.RottenProgress - 0.75f, 0f);
									}

									Mod.RegenerateWounds(l.SkinMaterialHandler, 5f);

									l.SkinMaterialHandler.Sync();
								}
							}
						}
						Instantiate<GameObject>(this.DisperseEffect, base.transform.position, Quaternion.identity);
						Destroy(base.gameObject);
					}
					public float Damage = -10f;
					public Rigidbody2D Rigidbody2D;
					public GameObject DisperseEffect;
					public float AcidChance = 0.2f;
				}
			}
		}
	}

	static class RandomExtensions
	{
		public static T[] Shuffle<T>(this System.Random rng, T[] array)
		{
			int n = array.Length;
			while (n > 1)
			{
				int k = rng.Next(n--);
				T temp = array[n];
				array[n] = array[k];
				array[k] = temp;
			}
			return array;
		}
	}
}

//Template by Polars Bear
