﻿using System;
using System.Collections.Generic;
using System.Reflection;
using UnityEngine;

namespace Mod
{
    class ShadowRealm : MonoBehaviour
    {
        public float timer;
        public GameObject obj;

        private void ChangeCollider(Transform parent, bool enable)
        {
            foreach (Transform child in parent)
            {
                if (child.gameObject.GetComponent<Collider2D>())
                    child.gameObject.GetComponent<Collider2D>().enabled = enable;
                ChangeCollider(child, enable);
            }
        }

        void Start()
        {
            ChangeCollider(obj.transform, false);
            obj.SetActive(false);
        }

        void OnDestroy()
        {
            ChangeCollider(obj.transform, true);
            obj.SetActive(true);
        }

        void Update()
        {
            timer -= Time.deltaTime;

            if (timer <= 0f)
            {
                Destroy(this);
            }
        }
    }

    public class GoAwaySyringe : SyringeBehaviour
    {
        public const string name = "​<b></b>" + "Shadow Realm Syringe";
        static GoAwaySyringeSerum liquid;
        float lodgedTime = 0f;
        bool lodged = false;

        public static void Load()
        {
            liquid = new GoAwaySyringeSerum();
            ModAPI.RegisterLiquid(GoAwaySyringeSerum.ID, liquid);

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),

                    NameOverride = name,
                    DescriptionOverride = "Syringe that sends people and things to the shadow realm.",

                    CategoryOverride = ModAPI.FindCategory("Polars Bear"),
                    AfterSpawn = (Instance) => { Destroy(Instance.GetComponent<SyringeBehaviour>()); Instance.GetOrAddComponent<GoAwaySyringe>(); }
                }
            );
            Mod.nailgun.SyringeNail(name, typeof(GoAwaySyringe));
        }

        List<GameObject> objects;

        void SearchLimb(LimbBehaviour l)
        {
            foreach (LimbBehaviour i in l.ConnectedLimbs)
            {
                if (!objects.Contains(i.gameObject))
                {
                    objects.Add(i.gameObject);
                    SearchLimb(i);
                }
            }
        }

        public override void Dislodged(PhysicalBehaviour.Penetration penetration)
        {
            lodged = false;
            GameObject obj = penetration.Victim.gameObject;
            while (obj.transform.parent)
            {
                obj = obj.transform.parent.gameObject;
            }
            if (obj.name == "WORLD") { return; }

            if (penetration.Victim.GetComponent<LimbBehaviour>())
            {

                ShadowRealm sr = gameObject.AddComponent<ShadowRealm>();
                sr.timer = lodgedTime;
                sr.obj = penetration.Victim.GetComponent<LimbBehaviour>().Person.gameObject;
            }
            else
            {
                ShadowRealm sr = gameObject.AddComponent<ShadowRealm>();
                sr.timer = lodgedTime;
                sr.obj = penetration.Victim.gameObject;
            }
            
        }

        void Update()
        {
            if (lodged)
            {
                lodgedTime += Time.deltaTime;
            }
        }

        public override void Lodged(Stabbing stabbing)
        {
            lodgedTime = 0f;
            lodged = true;
        }

        public override string GetLiquidID() => GoAwaySyringeSerum.ID;
        public class GoAwaySyringeSerum : libs.OneTimeBodyLiquid
        {
            public const string ID = "​<b></b>" + "Shadow Realm Serum";
            public GoAwaySyringeSerum()
            {
                Color = new Color(0f, 0f, 0f);
            }
            public override void OnEnterLimb(LimbBehaviour limb)
            {
                base.OnEnterLimb(limb);
                ShadowRealm sr = GameObject.Find("WORLD").AddComponent<ShadowRealm>();
                sr.timer = 0.2f;
                sr.obj = limb.Person.gameObject;
            }
            
        }

    }
}

//Author: Polars Bear
