﻿using System;
using System.Collections;
using System.Reflection;
using UnityEngine;

namespace Mod
{
    public class HeartAttackSyringe : SyringeBehaviour
    {
        public const string name = "​<b></b>" + "Heart Attack Syringe";
        public static void Load()
        {
            ModAPI.RegisterLiquid(HeartAttackSyringeSerum.ID, new HeartAttackSyringeSerum());

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),

                    NameOverride = name,
                    DescriptionOverride = "Syringe that causes a heart attack.",

                    CategoryOverride = ModAPI.FindCategory("Polars Bear"),
                    AfterSpawn = (Instance) => { Destroy(Instance.GetComponent<SyringeBehaviour>()); Instance.GetOrAddComponent<HeartAttackSyringe>(); }
                }
            );
            Mod.nailgun.SyringeNail(name, typeof(HeartAttackSyringe));
        }
        public override string GetLiquidID() => HeartAttackSyringeSerum.ID;
        public class HeartAttackSyringeSerum : libs.OneTimeBodyLiquid
        {
            public const string ID = "​<b></b>" + "COLESTEROL";
            public HeartAttackSyringeSerum()
            {
                Color = new Color(1f, 1.00000f, 0.6000f);
            }
            public override void OnEnterLimb(LimbBehaviour limb) 
            {
                if (limb.CirculationBehaviour.IsPump)
                {
                    foreach (LimbBehaviour limby in limb.Person.Limbs)
                    {
                        limby.gameObject.GetOrAddComponent<HeartPalpitations>();
                    }
                    limb.gameObject.GetOrAddComponent<Seizures>();
                }
            }
            public override void OnExitContainer(BloodContainer container) { }
            public override void OnUpdate(BloodContainer container) { }
            public override void OnEnterContainer(BloodContainer container) { }

            public class HeartPalpitations : MonoBehaviour
            {
                float timer;
                LimbBehaviour limb;

                void Start()
                {
                    limb = GetComponent<LimbBehaviour>();
                    timer = (6 - limb.DistanceToBrain) * 4;
                }

                void Update()
                {
                    if (limb.CirculationBehaviour.IsPump)
                    { limb.CirculationBehaviour.BloodFlow = 0.35f; }
                    timer -= Time.deltaTime;
                    if (timer <= 0)
                    {
                        limb.Health = 0f;
                        Destroy(this);
                    }
                }
            }

            public class Seizures : MonoBehaviour
            {
                float timer = 0f;
                PersonBehaviour person;
                void Start()
                {
                    person = GetComponent<LimbBehaviour>().Person;
                }

                void Update()
                {
                    if (!person.IsAlive())
                    {
                        Destroy(this);
                    }
                    timer += Time.deltaTime;

                    while (timer >= 0.5f)
                    {
                        timer -= 0.5f;
                        person.ShockLevel = UnityEngine.Random.Range(0.35f, 1f);
                        person.PainLevel = UnityEngine.Random.Range(0.35f, 1f);
                    }
                }
            }
        }

    }
}

//Author: Polars Bear
