﻿using System;
using System.Collections;
using System.Reflection;
using UnityEngine;

namespace Mod
{
    public class RadioactiveSyringe : SyringeBehaviour
    {
        public const string name = "​<b></b>" + "Radioactive Syringe";
        public LightSprite light;
        public Liquid liquid;
        float counter = 0f;
        public static void Load()
        {
            ModAPI.RegisterLiquid(RadioactiveSyringeSerum.ID, new RadioactiveSyringeSerum());

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),

                    NameOverride = name,
                    DescriptionOverride = "Syringe that contains highly radioactive materials.",

                    CategoryOverride = ModAPI.FindCategory("Polars Bear"),
                    AfterSpawn = (Instance) => { Destroy(Instance.GetComponent<SyringeBehaviour>()); Instance.GetOrAddComponent<RadioactiveSyringe>();}
                }
            );
            Mod.nailgun.SyringeNail(name, typeof (RadioactiveSyringe));
        }
        int? oldLightLayer;
        void Awake()
        {
            base.Awake();
            liquid = Liquid.GetLiquid(GetLiquidID());
            light = ModAPI.CreateLight(transform, liquid.Color, UnityEngine.Random.Range(0.3f,.5f), 1.5f);
            light.gameObject.layer = gameObject.layer;
        }


        void Update()
        {
            base.Update();
            counter += Time.deltaTime;
            if (counter > 0.1f)
            {
                light.Radius = UnityEngine.Random.Range(0.3f, .5f);
                counter = 0f;
            }
        }

        void OnRenderObject()
        {
            if (light.gameObject.layer != 15)
            {
                light.gameObject.layer = 15;
            }
        }

        public override string GetLiquidID() => RadioactiveSyringeSerum.ID;
        public class RadioactiveSyringeSerum : Liquid
        {
            public const string ID = "​<b></b>" + "HIGHLY RADIOACTIVE NEPTUNIUM";
            public RadioactiveSyringeSerum()
            {
                Color = new Color(0f, 1f, 0f);
            }
            public override void OnEnterLimb(LimbBehaviour limb)
            {

            }
            public override void OnExitContainer(BloodContainer container)
            {
                RadioactiveLightHolder rad;
                if (container.TryGetComponent<RadioactiveLightHolder>(out rad))
                {
                    Destroy(rad);
                }
            }
            public override void OnUpdate(BloodContainer container) { }
            public override void OnEnterContainer(BloodContainer container)
            {
                if (container.TryGetComponent<SyringeBehaviour>(out _)) { return; }
                container.gameObject.GetOrAddComponent<RadioactiveLightHolder>();
            }

            public class RadioactiveLightHolder : MonoBehaviour
            {
                public LightSprite light;
                Liquid liquid;
                float counter = 0f;
                PhysicalBehaviour phys;
                BloodContainer container;
                GameObject influence;
                CircleCollider2D influenceCol;
                RadioactiveDamager damager;

                void Awake()
                {
                    liquid = Liquid.GetLiquid(ID);
                    TryGetComponent(out phys);
                    container = GetComponent<BloodContainer>();
                    light = ModAPI.CreateLight(transform, liquid.Color, UnityEngine.Random.Range(0.75f, 1.5f) * transform.lossyScale.magnitude, 3.75f * (container.GetAmount(liquid) / container.TotalLiquidAmount));
                    /*influence = new GameObject("Radiation Influence");
                    influence.transform.parent = transform;
                    influence.transform.position = transform.position;
                    influenceCol = influence.AddComponent<CircleCollider2D>();
                    influenceCol.isTrigger = true;
                    damager = influence.AddComponent<RadioactiveDamager>();*/
                }

                void Update()
                {
                    float stuff = Utils.MapRange(container.Limits.x, container.Limits.y, 0f, 1f, container.GetAmount(liquid));
                    //influenceCol.radius = Utils.MapRange(container.Limits.x, container.Limits.y, 0f, 1f, container.GetAmount(liquid));
                    counter += Time.deltaTime;
                    if (counter > 0.1f)
                    {
                        light.Radius = UnityEngine.Random.Range(0.75f, 1.5f) * 2 * transform.lossyScale.magnitude;
                        light.Brightness = 4.75f * stuff;
                        counter = 0f;
                    }
                    if (phys)
                    {
                        phys.Temperature += Time.deltaTime * 15f * stuff;
                    }
                    /*ModAPI.Draw.Circle(influence.transform.position, stuff * 5f);
                    influenceCol.radius *= 1.5f;*/
                }

                void OnDestroy()
                {
                    Destroy(light.gameObject);
                }

                public class RadioactiveDamager : MonoBehaviour
                {
                    CircleCollider2D influence;
                    void Awake()
                    {
                        influence = GetComponent<CircleCollider2D>();
                    }

                    void OnTriggerEnter2D(Collider2D other)
                    {
                        Debug.Log(other);
                        LimbBehaviour limb;
                        if (other.TryGetComponent(out limb))
                        {
                            limb.CirculationBehaviour.StartCoroutine(RadDamagePB(limb,influence));
                        }
                    }

                    void OnTriggerExit2D(Collider2D other)
                    {
                        LimbBehaviour limb;
                        if (other.TryGetComponent(out limb))
                        {
                            limb.CirculationBehaviour.StopCoroutine("RadDamagePB");
                        }
                    }

                    private IEnumerator RadDamagePB(LimbBehaviour limb, CircleCollider2D col)
                    {
                        while (true)
                        {
                            float distance = Vector3.Distance(limb.transform.position,col.transform.position);
                            limb.Health -= (col.radius*2500f / distance) * Time.deltaTime;
                            limb.Health = Mathf.Clamp(limb.Health, 0f, 100f);
                            yield return new WaitForEndOfFrame();
                        }
                    }

                }
            }
        }

    }
}

//Author: Polars Bear
