﻿using System;
using System.Collections;
using System.Reflection;
using UnityEngine;

namespace Mod
{
    public class SlimeSyringe : SyringeBehaviour
    {
        public const string name = "​<b></b>" + "Slime Syringe";
        public static void Load()
        {
            ModAPI.RegisterLiquid(SlimeSyringeSerum.ID, new SlimeSyringeSerum());

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),

                    NameOverride = name,
                    DescriptionOverride = "Syringe that goops up the user.",

                    CategoryOverride = ModAPI.FindCategory("Polars Bear"),
                    AfterSpawn = (Instance) => { Destroy(Instance.GetComponent<SyringeBehaviour>()); Instance.GetOrAddComponent<SlimeSyringe>(); }
                }
            );

            Mod.nailgun.SyringeNail(name, typeof(SlimeSyringe));
        }

        public override string GetLiquidID() => SlimeSyringeSerum.ID;
        public class SlimeSyringeSerum : Liquid
        {
            public const string ID = "​<b></b>" + "SLIME";
            public SlimeSyringeSerum()
            {
                Color = new UnityEngine.Color(0.4f, 0.80000f, 0.2f);
            }
            public override void OnEnterLimb(LimbBehaviour limb)
            {
                limb.CirculationBehaviour.StartCoroutine(GooperatorPB(limb, Liquid.GetLiquid(limb.BloodLiquidType)));
                limb.BloodLiquidType = ID;
                limb.gameObject.GetOrAddComponent<StickyBehaviour>();
            }
            public override void OnExitContainer(BloodContainer container) { }
            public override void OnUpdate(BloodContainer container)
            {
                CirculationBehaviour circulationBehaviour = container as CirculationBehaviour;
                if (circulationBehaviour != null && circulationBehaviour.Limb.SpeciesIdentity == "Android" && UnityEngine.Random.value > 0.995f)
                {
                    circulationBehaviour.Limb.PhysicalBehaviour.Charge += 5f;
                }
            }
            public override void OnEnterContainer(BloodContainer container) { }
            
            private class StickyBehaviour : MonoBehaviour
            {
                void OnCollisionEnter2D(Collision2D col)
                {
                    for (int i = 0; i < col.contactCount; i++) {
                        FixedJoint2D joint = col.otherCollider.gameObject.GetOrAddComponent<FixedJoint2D>();
                        joint.connectedBody = col.rigidbody;
                        const float breaky = 5f * 10;
                        joint.breakForce = breaky;
                        joint.breakTorque = breaky;
                        joint.dampingRatio = 10f;
                        joint.frequency = 1f;
                        Debug.Log(joint.anchor);
                        joint.anchor = transform.InverseTransformPoint(col.GetContact(i).point);
                        joint.enableCollision = true;
                    }
                }
            }

            private IEnumerator GooperatorPB(LimbBehaviour limb, Liquid ogBlood)
            {
                float interp = 0f;
                Color startColor = limb.Color;
                while (interp < 1f)
                {
                    interp += 0.1f * Time.deltaTime;
                    limb.Color = Color.Lerp(startColor, Color, interp);
                    float amount = Math.Min(0.1f * Time.deltaTime, limb.CirculationBehaviour.GetAmount(ogBlood));
                    limb.CirculationBehaviour.RemoveLiquid(ogBlood, amount);
                    limb.CirculationBehaviour.AddLiquid(this, amount);
                    yield return new WaitForEndOfFrame();
                }
            }
        }

    }
}

//Author: Polars Bear
