﻿using System;
using System.Collections;
using System.Reflection;
using UnityEngine;

namespace Mod
{
    public class VampireSyringe : SyringeBehaviour
    {
        public const string name = "​<b></b>" + "Vampire Syringe";
        public static void Load()
        {
            ModAPI.RegisterLiquid(VampireSyringeSerum.ID, new VampireSyringeSerum());

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),

                    NameOverride = name,
                    DescriptionOverride = "Syringe that makes the user drink fluids and heal by doing so.",

                    CategoryOverride = ModAPI.FindCategory("Polars Bear"),
                    AfterSpawn = (Instance) => { Destroy(Instance.GetComponent<SyringeBehaviour>()); Instance.GetOrAddComponent<VampireSyringe>(); Instance.GetComponent<SpriteRenderer>().material.color = new Color(.65f, 0f, 0f); }
                }
            );
            Mod.nailgun.SyringeNail(name, typeof(VampireSyringe));
        }
        public override string GetLiquidID() => VampireSyringeSerum.ID;

        class VampireHead : MonoBehaviour
        {
            LimbBehaviour head;
            void Start()
            {
                GOTOHELLDEVELOPERS();
                head = GetComponent<LimbBehaviour>();
                head.Person.Wince(15f);
                GetComponent<SkinMaterialHandler>().renderer.color = new Color(0.85f, 0.65f, 0.65f);
            }

            void Heal(LimbBehaviour limby, float multiplier = 0.5f)
            {
                foreach (LimbBehaviour l in limby.Person.Limbs)
                {
                    if (!l.IsAndroid)
                    {
                        l.Health = Math.Min(l.InitialHealth, l.Health + 5f*Time.deltaTime*multiplier);
                        if (UnityEngine.Random.value > 0.25f/multiplier)
                        {
                            l.CirculationBehaviour.HealBleeding();
                            l.HealBone(); l.LungsPunctured = false;
                        }
                        l.Numbness = Math.Max(0f, l.Numbness - 0.75f * Time.deltaTime * multiplier);

                        l.CirculationBehaviour.IsPump = l.CirculationBehaviour.WasInitiallyPumping;
                        l.CirculationBehaviour.BloodFlow = Math.Min(1f, l.CirculationBehaviour.BloodFlow + 0.75f * Time.deltaTime * multiplier);
                        l.CirculationBehaviour.AddLiquid(l.GetOriginalBloodType(), Math.Max(Mathf.Min(0.75f * multiplier*Time.deltaTime, 1f - l.CirculationBehaviour.GetAmount(l.GetOriginalBloodType())), 0f));
                        l.BruiseCount = (ushort)Math.Max(0, l.BruiseCount - 1 * Time.deltaTime * multiplier);
                        l.CirculationBehaviour.GunshotWoundCount = (ushort)Math.Max(0, l.CirculationBehaviour.GunshotWoundCount - 1 * Time.deltaTime * multiplier);
                        l.CirculationBehaviour.StabWoundCount = (ushort)Math.Max(0, l.CirculationBehaviour.StabWoundCount - 1 * Time.deltaTime * multiplier);
                        if (l.RoughClassification == LimbBehaviour.BodyPart.Head)
                        {
                            l.Person.Consciousness = Math.Min(1f, l.Person.Consciousness + 0.75f * Time.deltaTime * multiplier);
                            l.Person.ShockLevel = Math.Max(0f, l.Person.ShockLevel - 0.75f * Time.deltaTime * multiplier);
                            l.Person.PainLevel = Math.Max(0f, l.Person.PainLevel - 0.75f * Time.deltaTime * multiplier);
                            l.Person.OxygenLevel = Math.Min(1f, l.Person.OxygenLevel + 0.75f * Time.deltaTime * multiplier); ;
                            l.Person.AdrenalineLevel = Math.Min(1f, l.Person.AdrenalineLevel + 0.75f * Time.deltaTime * multiplier); ;
                        }
                        if (l.PhysicalBehaviour.BurnProgress > 0)
                        {
                            l.PhysicalBehaviour.BurnProgress = Math.Max(l.PhysicalBehaviour.BurnProgress - 0.75f * Time.deltaTime * multiplier, 0f);
                        }
                        if (l.SkinMaterialHandler.AcidProgress > 0)
                        {
                            l.SkinMaterialHandler.AcidProgress = Math.Max(l.SkinMaterialHandler.AcidProgress - 0.75f * Time.deltaTime * multiplier, 0f);
                        }
                        if (l.SkinMaterialHandler.RottenProgress > 0)
                        {
                            l.SkinMaterialHandler.RottenProgress = Math.Max(l.SkinMaterialHandler.RottenProgress - 0.75f * Time.deltaTime * multiplier, 0f);
                        }

                        Mod.RegenerateWounds(l.SkinMaterialHandler, 10f * Time.deltaTime);

                        l.SkinMaterialHandler.Sync();
                    }
                }
            }
            void GOTOHELLDEVELOPERS()
            {
                Debug.Log("The next time you murder a method like this, I swear, you will suffer the same fate");
            }

            public virtual void OnDrink(BloodContainer container)
            {
                Heal(head);
                
            }
            void OnCollisionStay2D(Collision2D collision)
            {
                float amount = 5f;
                if (collision.collider.GetComponent<LimbBehaviour>())
                {
                    LimbBehaviour limb = collision.collider.GetComponent<LimbBehaviour>();
                    if (limb.Person == head.Person) { return; }
                    amount = 10f;
                }


                if (collision.collider.gameObject.GetComponent<BloodContainer>())
                {
                    BloodContainer container = collision.collider.GetComponent<BloodContainer>();
                    if (container.GetComponent<SyringeBehaviour>())
                    {
                        if (container.GetComponent<SyringeBehaviour>().Finite == false) { container.GetComponent<SyringeBehaviour>().Finite = true; }
                    }

                    container.TransferTo(amount*Time.deltaTime, head.CirculationBehaviour);

                    OnDrink(container);
                    
                }
            }
        }

        public class VampireSyringeSerum : libs.OneTimeBodyLiquid
        {
            public const string ID = "​<b></b>" + "VAMPIRE BLOOD";
            public VampireSyringeSerum()
            {
                Color = new Color(0.65f, 0f, 0f);
            }
            public override void OnEnterLimb(LimbBehaviour limb) 
            {
                limb.Person.Limbs[0].gameObject.GetOrAddComponent<VampireHead>();
            }
        }

    }
}

//Author: Polars Bear
